/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package p4uclient;

import java.io.IOException;

/**
 * アプリ全体を管理
 * @author tondol
 */
public class MainApp {
    static ClientManager client = null;
    static TaskListener  listen = null;
    static FileManager   files = null;
    static OptionManager option = null;
    static TaskManager   tasks = null;
    static MainView      window = null;
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        window = MainView.getInstance();
        option = new OptionManager("config.ini");
        files  = new FileManager();
        tasks  = new TaskManager();
    }

    /**
     * 終了時の処理
     */
    public static void exit() {
        try {
            offline();
            option.save();
            tasks.close();
        } catch(Exception e) {
            e.printStackTrace();
        } finally {
            System.exit(0);
        }
    }
    
    /**
     * 各セッションを開始
     */
    public static void online() {
        try {
            window.setBusy(true);
            client = new ClientManager();
            listen = new TaskListener();
            files.updateItems();
            window.setOnline(true);
            window.putMessage("接続成功 <" + option.host + ":" + option.port + ">");
        } catch(Exception e) {
            try {
                if (client != null) client.close();
                if (listen != null) listen.close();
            } catch(IOException ex) {
                e.printStackTrace();
            } finally {
                window.putMessage("接続失敗 <" + option.host + ":" + option.port + ">");
            }
        } finally {
            window.setBusy(false);
        }
    }
    
    /**
     * 各セッションを終了
     */
    public static void offline() {
        try {
            window.setBusy(true);
            window.setOnline(false);
            files.clearItems();
            if (client != null) client.close();
            if (listen != null) listen.close();
        } catch(Exception e) {
            e.printStackTrace();
        } finally {
            window.setBusy(false);
            window.putMessage("クライアント切断");
        }
    }
    
    /**
     * 共有リストを更新する操作
     */
    public static void update() {
        try {
            window.setBusy(true);
            files.updateItems();
            window.putMessage("リスト更新成功 <" + option.upDir + ">");
        } catch(Exception e) {
            window.putMessage("リスト更新失敗 <" + option.upDir + ">");
        } finally {
            window.setBusy(false);
        }
    }
    
    /**
     * サーバーの共有リストを検索
     */
    public static void search(String word) {
        try {
            if (word.length() == 0) word = ".";
            files.searchRemote(word);
            window.putMessage("検索完了 <" + word + ">");
        } catch(IOException e) {
            window.putMessage("検索失敗 <" + word + ">");
            offline();
        }
    }
    
    /**
     * ファイルを受信
     */
    public static void download(FileItem item) {
        try {
            TaskDownload task = new TaskDownload(item);
            tasks.addItem(task);
        } catch(Exception e) {
            e.printStackTrace();
        }
    }
    
    /**
     * タスクをキャンセル
     */
    public static void cancel(TaskAbstract task) {
        try {
            tasks.deleteItem(task);
        } catch(Exception e) {
            e.printStackTrace();
        }
    }
}
