/*
 * MainView.java
 *
 * Created on 2008/05/29, 18:01
 */

package p4uclient;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.UIManager;

/**
 * アプリのインターフェイスを管理
 * @author  tondol
 */
public class MainView extends javax.swing.JFrame {
    private static MainView instance = null;
    FileTableModel modelSearch       = null;
    TaskTableModel modelTasks        = null;
    // UIの状態を保持
    private boolean busy   = false;
    private boolean online = false;
    
    /** Creates new form MainView */
    private MainView() {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            SwingUtilities.updateComponentTreeUI(this);
        } catch(Exception e) {
            ;
        } finally {
            modelSearch = new FileTableModel();
            modelTasks  = new TaskTableModel();
            initComponents();
            changeState();
        }
    }
    
    /**
     * インスタンスを生成しつつ表示
     */
    public static MainView getInstance() {
        if (instance == null) {
            instance = new MainView();
            java.awt.EventQueue.invokeLater(new Runnable() {
                public void run() {
                    instance.setVisible(true);
                }
            });
        }
        return instance;
    }
    
    /**
     * UIの状態を変更
     */
    public void setBusy(boolean b) {
        busy = b;
        changeState();
    }
    public void setOnline(boolean b) {
        online = b;
        changeState();
    }
    public void changeState() {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                if (busy) {
                    btnOnline.setEnabled(false);
                    btnOffline.setEnabled(false);
                    btnUpdate.setEnabled(false);
                    txtSearch.setEnabled(false);
                    btnSearch.setEnabled(false);
                    btnDownload.setEnabled(false);
                    btnCancel.setEnabled(false);
                    btnClear.setEnabled(false);
                    return;
                }
                if (online) {
                    btnOnline.setEnabled(false);
                    btnOffline.setEnabled(true);
                    btnUpdate.setEnabled(true);
                    txtSearch.setEnabled(true);
                    btnSearch.setEnabled(true);
                    btnDownload.setEnabled(true);
                    btnCancel.setEnabled(true);
                    btnClear.setEnabled(true);
                } else {
                    btnOnline.setEnabled(true);
                    btnOffline.setEnabled(false);
                    btnUpdate.setEnabled(false);
                    txtSearch.setEnabled(false);
                    btnSearch.setEnabled(false);
                    btnDownload.setEnabled(false);
                    btnCancel.setEnabled(false);
                    btnClear.setEnabled(true);
                }
            }
        });
    }
    
    /**
     * ログにメッセージを書き出す
     */
    public void putMessage(String msg) {
        taOutput.append(msg + "\n");
        taOutput.setCaretPosition(taOutput.getText().length());
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        tabPane = new javax.swing.JTabbedPane();
        panelSearch = new javax.swing.JPanel();
        scrollSearch = new javax.swing.JScrollPane();
        tableSearch = new javax.swing.JTable();
        txtSearch = new javax.swing.JTextField();
        btnSearch = new javax.swing.JButton();
        btnDownload = new javax.swing.JButton();
        panelTasks = new javax.swing.JPanel();
        scrollTasks = new javax.swing.JScrollPane();
        tableTasks = new javax.swing.JTable();
        btnCancel = new javax.swing.JButton();
        panelLog = new javax.swing.JPanel();
        scrollLog = new javax.swing.JScrollPane();
        taOutput = new javax.swing.JTextArea();
        btnClear = new javax.swing.JButton();
        btnOnline = new javax.swing.JButton();
        btnOffline = new javax.swing.JButton();
        btnConfig = new javax.swing.JButton();
        btnUpdate = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("P4UClient");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        tabPane.setFont(tabPane.getFont().deriveFont(tabPane.getFont().getSize()+1f));
        tabPane.setMinimumSize(new java.awt.Dimension(100, 100));
        tabPane.setPreferredSize(new java.awt.Dimension(200, 150));

        tableSearch.setFont(tableSearch.getFont().deriveFont(tableSearch.getFont().getSize()+2f));
        tableSearch.setModel(this.modelSearch);
        tableSearch.setOpaque(false);
        tableSearch.setRowHeight(18);
        tableSearch.setRowMargin(2);
        tableSearch.getTableHeader().setReorderingAllowed(false);
        tableSearch.setVerifyInputWhenFocusTarget(false);
        scrollSearch.setViewportView(tableSearch);

        txtSearch.setFont(txtSearch.getFont().deriveFont(txtSearch.getFont().getSize()+2f));
        txtSearch.setMargin(new java.awt.Insets(3, 3, 3, 3));
        txtSearch.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                searchAction(evt);
            }
        });
        txtSearch.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                slectTextAction(evt);
            }
        });

        btnSearch.setText("検索");
        btnSearch.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                searchAction(evt);
            }
        });

        btnDownload.setText("取得");
        btnDownload.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                downloadAction(evt);
            }
        });

        javax.swing.GroupLayout panelSearchLayout = new javax.swing.GroupLayout(panelSearch);
        panelSearch.setLayout(panelSearchLayout);
        panelSearchLayout.setHorizontalGroup(
            panelSearchLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelSearchLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelSearchLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(panelSearchLayout.createSequentialGroup()
                        .addComponent(txtSearch, javax.swing.GroupLayout.DEFAULT_SIZE, 374, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnSearch)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnDownload))
                    .addComponent(scrollSearch, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 516, Short.MAX_VALUE))
                .addContainerGap())
        );
        panelSearchLayout.setVerticalGroup(
            panelSearchLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelSearchLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelSearchLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(txtSearch, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(btnSearch)
                    .addComponent(btnDownload))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(scrollSearch, javax.swing.GroupLayout.DEFAULT_SIZE, 223, Short.MAX_VALUE)
                .addContainerGap())
        );

        tabPane.addTab(" 検索 ", panelSearch);

        tableTasks.setFont(tableTasks.getFont().deriveFont(tableTasks.getFont().getSize()+2f));
        tableTasks.setModel(this.modelTasks);
        tableTasks.setOpaque(false);
        tableTasks.setRowHeight(18);
        tableTasks.setRowMargin(2);
        scrollTasks.setViewportView(tableTasks);

        btnCancel.setText("中断");
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelAction(evt);
            }
        });

        javax.swing.GroupLayout panelTasksLayout = new javax.swing.GroupLayout(panelTasks);
        panelTasks.setLayout(panelTasksLayout);
        panelTasksLayout.setHorizontalGroup(
            panelTasksLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panelTasksLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelTasksLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(scrollTasks, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 516, Short.MAX_VALUE)
                    .addComponent(btnCancel))
                .addContainerGap())
        );
        panelTasksLayout.setVerticalGroup(
            panelTasksLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panelTasksLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(scrollTasks, javax.swing.GroupLayout.DEFAULT_SIZE, 225, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnCancel)
                .addContainerGap())
        );

        tabPane.addTab(" タスク ", panelTasks);

        taOutput.setColumns(20);
        taOutput.setEditable(false);
        taOutput.setLineWrap(true);
        taOutput.setRows(5);
        scrollLog.setViewportView(taOutput);

        btnClear.setText("クリア");
        btnClear.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearAction(evt);
            }
        });

        javax.swing.GroupLayout panelLogLayout = new javax.swing.GroupLayout(panelLog);
        panelLog.setLayout(panelLogLayout);
        panelLogLayout.setHorizontalGroup(
            panelLogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panelLogLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelLogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(scrollLog, javax.swing.GroupLayout.DEFAULT_SIZE, 516, Short.MAX_VALUE)
                    .addComponent(btnClear))
                .addContainerGap())
        );
        panelLogLayout.setVerticalGroup(
            panelLogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panelLogLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(scrollLog, javax.swing.GroupLayout.DEFAULT_SIZE, 225, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnClear)
                .addContainerGap())
        );

        tabPane.addTab(" ログ ", panelLog);

        btnOnline.setText("接続");
        btnOnline.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mainActions(evt);
            }
        });

        btnOffline.setText("切断");
        btnOffline.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mainActions(evt);
            }
        });

        btnConfig.setText("設定");
        btnConfig.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                configAction(evt);
            }
        });

        btnUpdate.setText("スキャン");
        btnUpdate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mainActions(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(tabPane, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 549, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(btnOnline)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnOffline)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnUpdate)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 253, Short.MAX_VALUE)
                        .addComponent(btnConfig)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnOnline)
                    .addComponent(btnConfig)
                    .addComponent(btnOffline)
                    .addComponent(btnUpdate))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(tabPane, javax.swing.GroupLayout.DEFAULT_SIZE, 324, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        java.awt.Dimension dialogSize = getSize();
        setLocation((screenSize.width-dialogSize.width)/2,(screenSize.height-dialogSize.height)/2);
    }// </editor-fold>//GEN-END:initComponents

    /**
     * 基本的なアクション
     */
    private void mainActions(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mainActions
        final String action = evt.getActionCommand();
        SwingWorker worker = new SwingWorker() {
            @Override protected Object doInBackground() {
                if (action.equals("接続")) {
                    MainApp.online();
                } else if (action.equals("スキャン")) {
                    MainApp.update();
                } else if (action.equals("切断")) {
                    MainApp.offline();
                }
                return null;
        }};
        worker.execute();
}//GEN-LAST:event_mainActions

    /**
     * テキストボックスを選択
     */
    private void slectTextAction(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_slectTextAction
        javax.swing.JTextField textBox = (javax.swing.JTextField)evt.getSource();
        textBox.selectAll();
}//GEN-LAST:event_slectTextAction

    /**
     * ファイルを検索する
     */
    private void searchAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_searchAction
        final String word = txtSearch.getText();
        SwingWorker worker = new SwingWorker() {
            @Override protected Object doInBackground() {
                MainApp.search(word);
                return null;
        }};
        worker.execute();
}//GEN-LAST:event_searchAction

    /**
     * 選択したファイルをダウンロード
     */
    private void downloadAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_downloadAction
        // 選択チェック
        int select = tableSearch.getSelectedRow();
        if (select == -1) return;
        final FileItem item = modelSearch.getItem(select);
        // 実行
        SwingWorker worker = new SwingWorker() {
            @Override protected Object doInBackground() {
                MainApp.download(item);
                return null;
        }};
        worker.execute();
}//GEN-LAST:event_downloadAction

    /**
     * ウィンドウを閉じる
     */
    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        int option = JOptionPane.showConfirmDialog(
                this, "本当に終了しちゃいますよ？",
                "P4UClient - 確認", JOptionPane.YES_NO_OPTION);
        if (option != JOptionPane.YES_OPTION) return;
        MainApp.exit();
    }//GEN-LAST:event_formWindowClosing

    /**
     * ログをクリア
     */
    private void clearAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearAction
        int option = JOptionPane.showConfirmDialog(
                this, "ログをクリアしちゃいますよ？",
                "P4UClient - 確認", JOptionPane.YES_NO_OPTION);
        if (option != JOptionPane.YES_OPTION) return;
        taOutput.setText("");
}//GEN-LAST:event_clearAction

    /**
     * タスクを中断
     */
    private void cancelAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelAction
        // 選択チェック
        int select = tableTasks.getSelectedRow();
        if (select == -1) return;
        final TaskAbstract item = modelTasks.getItem(select);
        // ダイアログで確認
        int option = JOptionPane.showConfirmDialog(
                this, "タスクを中断しちゃいますよ？",
                "P4UClient - 確認", JOptionPane.YES_NO_OPTION);
        if (option != JOptionPane.YES_OPTION) return;
        // 実行
        SwingWorker worker = new SwingWorker() {
            @Override
            protected Object doInBackground() {
                MainApp.cancel(item);
                return null;
        }};
}//GEN-LAST:event_cancelAction

    /**
     * 設定ダイアログを開く
     */
    private void configAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_configAction
        OptionBox.showOptionBox();
    }//GEN-LAST:event_configAction

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnClear;
    private javax.swing.JButton btnConfig;
    private javax.swing.JButton btnDownload;
    private javax.swing.JButton btnOffline;
    private javax.swing.JButton btnOnline;
    private javax.swing.JButton btnSearch;
    private javax.swing.JButton btnUpdate;
    private javax.swing.JPanel panelLog;
    private javax.swing.JPanel panelSearch;
    private javax.swing.JPanel panelTasks;
    private javax.swing.JScrollPane scrollLog;
    private javax.swing.JScrollPane scrollSearch;
    private javax.swing.JScrollPane scrollTasks;
    private javax.swing.JTextArea taOutput;
    private javax.swing.JTabbedPane tabPane;
    private javax.swing.JTable tableSearch;
    private javax.swing.JTable tableTasks;
    private javax.swing.JTextField txtSearch;
    // End of variables declaration//GEN-END:variables
}
